import bpy
from mathutils import Vector

class FIRE_OT_custom(bpy.types.Operator):
    """Draws straight curve between camera and empty"""
    bl_idname = "blenderbob.fire"
    bl_label = "Select Empty and Fire!"
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        sel_objs = bpy.context.selected_objects
        first_sel_empty = None
        
        for obj in sel_objs:
            if obj.type == 'EMPTY':
                first_sel_empty = obj
                break
        return first_sel_empty is not None

    def execute(self, context):
        maincam = bpy.context.scene.camera
        sel_objs = bpy.context.selected_objects
        first_sel_empty = None
        
        for obj in sel_objs:
            if obj.type == 'EMPTY':
                first_sel_empty = obj
                break
                
        if not first_sel_empty:
            raise RuntimeError('No empty selected')
        objs_to_delete = []
        
        for obj in bpy.context.scene.objects:
            if obj.name.startswith('CamToEmptyCurve'):
                objs_to_delete.append(obj)
                
        for object_to_delete in objs_to_delete:
            bpy.data.objects.remove(object_to_delete, do_unlink=True)
        new_curve = bpy.data.curves.new('path_curve', type='CURVE')
        new_curve.dimensions = '3D'
        path = new_curve.splines.new('POLY')

        path.points.add(2)
        path.points[0].co = Vector((0, 0, 0, 1))
        main_cam_translation = maincam.matrix_world.to_translation()
        vec_e = Vector((first_sel_empty.location.x, first_sel_empty.location.y, first_sel_empty.location.z, 1))
        vec_c = Vector((main_cam_translation.x, main_cam_translation.y, main_cam_translation.z, 1))
        path.points[1].co = (vec_e - vec_c) * 1000

        curve_obj = bpy.data.objects.new('CamToEmptyCurve', new_curve)
        curve_obj.location = main_cam_translation
        scene = bpy.context.scene
        scene.collection.objects.link(curve_obj)
        bpy.context.view_layer.objects.active = curve_obj
        curve_obj.select_set(True)
        bpy.ops.object.convert(target='MESH')
        bpy.context.view_layer.objects.active = first_sel_empty
        curve_obj.select_set(False)
        first_sel_empty.select_set(True)
        return {'FINISHED'}

class CREATE_EMPTY_OT_custom(bpy.types.Operator):
    """Creates an empty in the scene"""
    bl_idname = "blenderbob.create_empty"
    bl_label = "Create Empty"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        bpy.ops.object.empty_add(type='PLAIN_AXES')
        return {'FINISHED'}

class FIRE_PT_custom(bpy.types.Panel):
    """Creates a Sub-Panel in the Property Area of the 3D View"""
    bl_label = "Fire Ray onto Empty"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_idname = "FIRE_PT_custom"
    bl_category = "Tool"

    def draw(self, context):
        layout = self.layout
        row = layout.row()
        row.operator(CREATE_EMPTY_OT_custom.bl_idname, icon="EMPTY_AXIS")
        row = layout.row()
        row.operator(FIRE_OT_custom.bl_idname, icon="FORWARD")


def register():
    bpy.utils.register_class(FIRE_PT_custom)
    bpy.utils.register_class(FIRE_OT_custom)
    bpy.utils.register_class(CREATE_EMPTY_OT_custom)

def unregister():
    bpy.utils.unregister_class(FIRE_OT_custom)
    bpy.utils.unregister_class(CREATE_EMPTY_OT_custom)
    bpy.utils.unregister_class(FIRE_PT_custom)

if __name__ == "__main__":
    register()